// grapheditView.cpp : implementation of the CGrapheditView class
//

#include "stdafx.h"
#include "graphedit.h"

#include "grapheditDoc.h"
#include "grapheditView.h"
#include "drawcore.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void CGrapheditView::drag_arc()
{
	CDC * pdc = GetDC();
	dc.drag_arc(pdc);
	ReleaseDC(pdc);
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView

IMPLEMENT_DYNCREATE(CGrapheditView, CView)

BEGIN_MESSAGE_MAP(CGrapheditView, CView)
	//{{AFX_MSG_MAP(CGrapheditView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_MODE_INSERT_N, OnModeInsertN)
	ON_UPDATE_COMMAND_UI(ID_MODE_INSERT_N, OnUpdateModeInsertN)
	ON_COMMAND(ID_MODE_INSERT_ARC, OnModeInsertArc)
	ON_UPDATE_COMMAND_UI(ID_MODE_INSERT_ARC, OnUpdateModeInsertArc)
	ON_COMMAND(ID_MODE_EDIT, OnModeEdit)
	ON_UPDATE_COMMAND_UI(ID_MODE_EDIT, OnUpdateModeEdit)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView construction/destruction

CGrapheditView::CGrapheditView()
{
}

CGrapheditView::~CGrapheditView()
{
}

BOOL CGrapheditView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView drawing

void CGrapheditView::OnDraw(CDC* pDC)
{
	CGrapheditDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	dc.OnDraw(pDC,pDoc->nodes,pDoc->arcs);
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView printing

BOOL CGrapheditView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGrapheditView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGrapheditView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView diagnostics

#ifdef _DEBUG
void CGrapheditView::AssertValid() const
{
	CView::AssertValid();
}

void CGrapheditView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGrapheditDoc* CGrapheditView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGrapheditDoc)));
	return (CGrapheditDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView message handlers

void CGrapheditView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CGrapheditDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	//dc.OnLButtonDown(nFlags,point);
	
	switch(dc.state)
	{
	case DrawCore::S_INSERT_NODE:
		pDoc->add_node(point);
		break;
	case DrawCore::S_INSERT_ARC:
		{
			dc.start_node = pDoc->FindNode(point);
			if(dc.start_node!=-1)
			{
				dc.end_node = dc.start_node;
				dc.start_drag = dc.end_drag = pDoc->nodes[dc.start_node].p;  
				CDC * pdc = GetDC();
				dc.drag_arc(pdc);
				ReleaseDC(pdc);
				SetCapture();
				dc.captured = true;
			}
		}
		break;
	}
}

void CGrapheditView::OnModeInsertN() 
{
	dc.state = DrawCore::S_INSERT_NODE;	
}

void CGrapheditView::OnUpdateModeInsertN(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
	pCmdUI->SetCheck(dc.state == DrawCore::S_INSERT_NODE);
}

void CGrapheditView::OnModeInsertArc() 
{
	dc.state = DrawCore::S_INSERT_ARC;
}

void CGrapheditView::OnUpdateModeInsertArc(CCmdUI* pCmdUI) 
{
	CGrapheditDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->Enable(pDoc->nodes.size()>1);
	pCmdUI->SetCheck(dc.state == DrawCore::S_INSERT_ARC);
}

void CGrapheditView::OnModeEdit() 
{
	dc.state = DrawCore::S_EDIT;
}

void CGrapheditView::OnUpdateModeEdit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
	pCmdUI->SetCheck(dc.state == DrawCore::S_EDIT);
}

void CGrapheditView::OnMouseMove(UINT nFlags, CPoint point) 
{
	dc.OnMouseMove(nFlags,point);
	
	if(dc.captured)
	{
		CDC * pdc = GetDC();
		dc.drag_arc(pdc);
		dc.end_drag = point;
		dc.drag_arc(pdc);
		ReleaseDC(pdc);
	}
}

void CGrapheditView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	dc.OnLButtonUp(nFlags,point);
	if(dc.captured)
	{
		CGrapheditDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		CDC * pdc = GetDC();
		dc.drag_arc(pdc);
		ReleaseDC(pdc);
		ReleaseCapture();
		dc.captured = false;
		dc.end_node = pDoc->FindNode(point);
		if( dc.end_node!=-1 && dc.end_node!=dc.start_node)
		{
			pDoc->add_arc(dc.start_node,dc.end_node);
		}
	}
}
